import datetime
import functools
import math
from collections.abc import Iterable, Sequence
from enum import StrEnum, auto
from typing import Any, Generic, Literal, Never, Protocol, Self

import requests
from pydantic import (
    ConfigDict,
    Field,
    RootModel,
    TypeAdapter,
)
from typing_extensions import TypeVar

from unifi_access.schemas import (
    AccessPolicy,
    AccessPolicyId,
    ActorId,
    Device,
    DeviceId,
    Door,
    DoorEmergencyStatus,
    DoorGroup,
    DoorGroupId,
    DoorGroupTopology,
    DoorId,
    DoorLockingRule,
    DoorLockingRuleType,
    EnrollNfcCardResponse,
    FetchAllHolidayGroupsResponse,
    FetchAllSchedulesResponse,
    FetchAllVisitorsExpansion,
    FullUser,
    HolidayGroup,
    HolidayGroupId,
    IdentityInvitationEmailFailure,
    IdentityInvitationUser,
    NamedResource,
    NfcCard,
    NfcCardEnrollmentSessionId,
    NfcCardEnrollmentStatus,
    NfcCardToken,
    PartialHoliday,
    Resource,
    ResourceId,
    Schedule,
    ScheduleId,
    SystemLogTopic,
    TimePeriod,
    User,
    UserGroup,
    UserGroupId,
    UserId,
    UserStatus,
    Visitor,
    VisitorId,
    VisitorStatus,
    VisitReason,
    WeekSchedule,
)
from unifi_access.schemas._base import (
    ForbidExtraBaseModel,
    IdentityResourceId,
    UnixTimestampDateTime,
)
from unifi_access.schemas._identity import IdentityResource, IdentityResourceType
from unifi_access.schemas._system_log import FetchSystemLogsResponse, SystemLogEntry


class ResponseCode(StrEnum):
    @staticmethod
    def _generate_next_value_(
        name: str, start: int, count: int, last_values: list[str]
    ) -> str:
        return f"CODE_{name}"

    SUCCESS = "SUCCESS"

    PARAMS_INVALID = auto()
    SYSTEM_ERROR = auto()
    RESOURCE_NOT_FOUND = auto()
    OPERATION_FORBIDDEN = auto()
    AUTH_FAILED = auto()
    ACCESS_TOKEN_INVALID = auto()
    UNAUTHORIZED = auto()
    NOT_EXISTS = auto()
    USER_EMAIL_ERROR = auto()
    USER_ACCOUNT_NOT_EXIST = auto()
    USER_WORKER_NOT_EXISTS = auto()
    ADMIN_EMAIL_EXIST = auto()  # NOTE: undocumented
    USER_NAME_DUPLICATED = auto()
    USER_EMPLOYEE_NUMBER_EXIST = auto()  # NOTE: undocumented
    USER_CSV_IMPORT_INCOMPLETE_PROP = auto()
    ACCESS_POLICY_USER_TIMEZONE_NOT_FOUND = auto()
    ACCESS_POLICY_HOLIDAY_TIMEZONE_NOT_FOUND = auto()
    ACCESS_POLICY_HOLIDAY_GROUP_NOT_FOUND = auto()
    ACCESS_POLICY_HOLIDAY_NOT_FOUND = auto()
    ACCESS_POLICY_SCHEDULE_NOT_FOUND = auto()
    ACCESS_POLICY_HOLIDAY_NAME_EXIST = auto()
    ACCESS_POLICY_HOLIDAY_GROUP_NAME_EXIST = auto()
    ACCESS_POLICY_SCHEDULE_NAME_EXIST = auto()
    ACCESS_POLICY_SCHEDULE_CAN_NOT_DELETE = auto()
    ACCESS_POLICY_HOLIDAY_GROUP_CAN_NOT_DELETE = auto()
    CREDS_NFC_HAS_BIND_USER = auto()
    CREDS_DISABLE_TRANSFER_UID_USER_NFC = auto()
    CREDS_NFC_READ_SESSION_NOT_FOUND = auto()
    CREDS_NFC_READ_POLL_TOKEN_EMPTY = auto()
    CREDS_NFC_CARD_IS_PROVISION = auto()
    CREDS_NFC_CARD_PROVISION_FAILED = auto()
    CREDS_NFC_CARD_INVALID = auto()
    CREDS_NFC_CARD_CANNOT_BE_DELETE = auto()
    CREDS_PIN_CREDS_ALREADY_EXIST = auto()
    CREDS_PIN_CREDS_LENGTH_INVALID = auto()
    SPACE_DEVICE_BOUND_LOCATION_NOT_FOUND = auto()
    DEVICE_DEVICE_VERSION_NOT_FOUND = auto()
    DEVICE_DEVICE_VERSION_TOO_OLD = auto()
    DEVICE_DEVICE_BUSY = auto()
    DEVICE_DEVICE_NOT_FOUND = auto()
    DEVICE_DEVICE_OFFLINE = auto()
    OTHERS_UID_ADOPTED_NOT_SUPPORTED = auto()
    HOLIDAY_GROUP_CAN_NOT_DELETE = auto()
    HOLIDAY_GROUP_CAN_NOT_EDIT = auto()
    DEVICE_WEBHOOK_ENDPOINT_DUPLICATED = auto()


class UnifiAccessError(Exception):
    """Exception version of `ErrorResponse`"""

    def __init__(self, code: ResponseCode, msg: str) -> None:
        self.code = code
        self.msg = msg

        super().__init__(code, msg)


class ErrorResponse(ForbidExtraBaseModel):
    """A response with an error code, and no data"""

    code: Literal[
        ResponseCode.PARAMS_INVALID,
        ResponseCode.SYSTEM_ERROR,
        ResponseCode.RESOURCE_NOT_FOUND,
        ResponseCode.OPERATION_FORBIDDEN,
        ResponseCode.AUTH_FAILED,
        ResponseCode.ACCESS_TOKEN_INVALID,
        ResponseCode.UNAUTHORIZED,
        ResponseCode.NOT_EXISTS,
        ResponseCode.ADMIN_EMAIL_EXIST,
        ResponseCode.USER_EMAIL_ERROR,
        ResponseCode.USER_ACCOUNT_NOT_EXIST,
        ResponseCode.USER_WORKER_NOT_EXISTS,
        ResponseCode.USER_NAME_DUPLICATED,
        ResponseCode.USER_EMPLOYEE_NUMBER_EXIST,
        ResponseCode.USER_CSV_IMPORT_INCOMPLETE_PROP,
        ResponseCode.ACCESS_POLICY_USER_TIMEZONE_NOT_FOUND,
        ResponseCode.ACCESS_POLICY_HOLIDAY_TIMEZONE_NOT_FOUND,
        ResponseCode.ACCESS_POLICY_HOLIDAY_GROUP_NOT_FOUND,
        ResponseCode.ACCESS_POLICY_HOLIDAY_NOT_FOUND,
        ResponseCode.ACCESS_POLICY_SCHEDULE_NOT_FOUND,
        ResponseCode.ACCESS_POLICY_HOLIDAY_NAME_EXIST,
        ResponseCode.ACCESS_POLICY_HOLIDAY_GROUP_NAME_EXIST,
        ResponseCode.ACCESS_POLICY_SCHEDULE_NAME_EXIST,
        ResponseCode.ACCESS_POLICY_SCHEDULE_CAN_NOT_DELETE,
        ResponseCode.ACCESS_POLICY_HOLIDAY_GROUP_CAN_NOT_DELETE,
        ResponseCode.CREDS_NFC_HAS_BIND_USER,
        ResponseCode.CREDS_DISABLE_TRANSFER_UID_USER_NFC,
        ResponseCode.CREDS_NFC_READ_SESSION_NOT_FOUND,
        ResponseCode.CREDS_NFC_READ_POLL_TOKEN_EMPTY,
        ResponseCode.CREDS_NFC_CARD_IS_PROVISION,
        ResponseCode.CREDS_NFC_CARD_PROVISION_FAILED,
        ResponseCode.CREDS_NFC_CARD_INVALID,
        ResponseCode.CREDS_NFC_CARD_CANNOT_BE_DELETE,
        ResponseCode.CREDS_PIN_CREDS_ALREADY_EXIST,
        ResponseCode.CREDS_PIN_CREDS_LENGTH_INVALID,
        ResponseCode.SPACE_DEVICE_BOUND_LOCATION_NOT_FOUND,
        ResponseCode.DEVICE_DEVICE_VERSION_NOT_FOUND,
        ResponseCode.DEVICE_DEVICE_VERSION_TOO_OLD,
        ResponseCode.DEVICE_DEVICE_BUSY,
        ResponseCode.DEVICE_DEVICE_NOT_FOUND,
        ResponseCode.DEVICE_DEVICE_OFFLINE,
        ResponseCode.OTHERS_UID_ADOPTED_NOT_SUPPORTED,
        ResponseCode.HOLIDAY_GROUP_CAN_NOT_DELETE,
        ResponseCode.HOLIDAY_GROUP_CAN_NOT_EDIT,
        ResponseCode.DEVICE_WEBHOOK_ENDPOINT_DUPLICATED,
    ]
    msg: str
    # sometimes the errors include this
    data: Any | None = None

    def success_or_raise(self) -> Never:
        raise UnifiAccessError(self.code, self.msg)


class ResponsePagination(ForbidExtraBaseModel):
    page_num: int
    page_size: int
    total: int


# TODO: this has nicer syntax in Python 3.12, but not currently supported in Pydantic
ResponseDataType = TypeVar("ResponseDataType")
ResponsePaginationType = TypeVar("ResponsePaginationType", default=None)


class SuccessResponse(
    ForbidExtraBaseModel, Generic[ResponseDataType, ResponsePaginationType]
):
    """A successful response containing data"""

    code: Literal[ResponseCode.SUCCESS]
    msg: str
    # sometimes the Access API omits this when it would be null
    data: ResponseDataType = Field(default=None, validate_default=True)  # type: ignore
    pagination: ResponsePaginationType = Field(default=None, validate_default=True)  # type: ignore

    def success_or_raise(self) -> Self:
        return self


class BaseResponse(
    RootModel[
        SuccessResponse[ResponseDataType, ResponsePaginationType] | ErrorResponse
    ],
    Generic[ResponseDataType, ResponsePaginationType],
):
    @classmethod
    def validate_and_unwrap(cls, r: requests.Response) -> ResponseDataType:
        return cls.model_validate_json(r.content).root.success_or_raise().data


class PaginatedResponse(BaseResponse[ResponseDataType, ResponsePagination]):
    pass


# TODO: this really shouldn't be necessary, but the default on ResponsePaginationType doesn't seem
#       to work well through the RootModel in 3.11
class Response(BaseResponse[ResponseDataType, None]):
    pass


T = TypeVar("T")


class PageNumberable(Protocol, Generic[T]):
    def __call__(
        self, page_num: int
    ) -> SuccessResponse[list[T], ResponsePagination]: ...


def iterate_pages(fxn: PageNumberable[T]) -> Iterable[T]:
    resp = fxn(page_num=1)
    yield from resp.data

    total_pages = math.ceil(resp.pagination.total / resp.pagination.page_size)
    for page_num in range(2, total_pages + 1):
        yield from fxn(page_num=page_num).data


class RequestPagination(ForbidExtraBaseModel):
    page_num: int | None = None
    page_size: int | None = None


# used for both User and Visitor
class AssignNfcCardRequest(ForbidExtraBaseModel):
    token: NfcCardToken
    """Token of the NFC card."""
    force_add: bool | None = None
    """Determine whether to overwrite an NFC card that has already been assigned."""


class AccessClient:
    """Client for the UniFi Access API"""

    def __init__(self, host: str, api_token: str, verify: bool = True) -> None:
        self._base_url = f"https://{host}/api/v1/developer"

        self._session = requests.Session()
        self._session.headers.update(
            {
                "Authorization": f"Bearer {api_token}",
                "Accept": "application/json",
                "Content-Type": "application/json",
            }
        )
        self._session.verify = verify

    def register_user(
        self,
        first_name: str,
        last_name: str,
        user_email: str | None = None,
        employee_number: str | None = None,
        onboard_time: UnixTimestampDateTime | None = None,
    ) -> User:
        """3.2 User Registration"""

        class RegisterUserRequest(ForbidExtraBaseModel):
            first_name: str
            last_name: str
            user_email: str | None = None
            employee_number: str | None = None
            onboard_time: UnixTimestampDateTime | None = None

        body = RegisterUserRequest(
            first_name=first_name,
            last_name=last_name,
            user_email=user_email,
            employee_number=employee_number,
            onboard_time=onboard_time,
        ).model_dump(exclude_none=True)

        r = self._session.post(f"{self._base_url}/users", json=body)
        # TODO: not the complete User
        return Response[User].validate_and_unwrap(r)

    def update_user(  # noqa: PLR0913
        self,
        user_id: UserId,
        first_name: str | None = None,
        last_name: str | None = None,
        user_email: str | None = None,
        employee_number: str | None = None,
        onboard_time: UnixTimestampDateTime | None = None,
        status: UserStatus | None = None,
    ) -> None:
        """3.3 Update User"""

        class UpdateUserRequest(ForbidExtraBaseModel):
            first_name: str | None = None
            last_name: str | None = None
            user_email: str | None = None
            employee_number: str | None = None
            onboard_time: UnixTimestampDateTime | None = None
            status: UserStatus | None = None

        body = UpdateUserRequest(
            first_name=first_name,
            last_name=last_name,
            user_email=user_email,
            employee_number=employee_number,
            onboard_time=onboard_time,
            status=status,
        ).model_dump(exclude_none=True)

        r = self._session.put(f"{self._base_url}/users/{user_id}", json=body)
        # TODO: maybe not the complete User
        return Response[None].validate_and_unwrap(r)

    def fetch_user(
        self, user_id: UserId, expand_access_policies: bool = False
    ) -> FullUser:
        """3.4 Fetch User"""
        params = {"expand[]": "access_policy"} if expand_access_policies else {}

        r = self._session.get(f"{self._base_url}/users/{user_id}", params=params)
        return Response[FullUser].validate_and_unwrap(r)

    def fetch_all_users(
        self,
        expand_access_policies: bool = False,
        page_num: int | None = None,
        page_size: int | None = None,
    ) -> SuccessResponse[list[FullUser], ResponsePagination]:
        """3.5 Fetch All Users

        If you don't need to manually handle pagination, consider [`fetch_all_users__unpaged`][unifi_access.AccessClient.fetch_all_users__unpaged].
        """

        class FetchAllUsersParams(RequestPagination):
            model_config = ConfigDict(populate_by_name=True)
            expand: list[str] | None = Field(alias=str("expand[]"), default=None)  # noqa: UP018

        params = FetchAllUsersParams(
            expand=["access_policy"] if expand_access_policies else None,
            page_num=page_num,
            page_size=page_size,
        ).model_dump(exclude_none=True, by_alias=True)

        r = self._session.get(f"{self._base_url}/users", params=params)
        return (
            PaginatedResponse[list[FullUser]].model_validate_json(r.content).root
        ).success_or_raise()

    def fetch_all_users__unpaged(
        self,
        expand_access_policies: bool = False,
        page_size: int | None = None,
    ) -> Iterable[FullUser]:
        """3.5 Fetch All Users

        This will automatically handle pagination.
        If you need more control, consider [`fetch_all_users`][unifi_access.AccessClient.fetch_all_users].
        """
        yield from iterate_pages(
            functools.partial(
                self.fetch_all_users,
                expand_access_policies=expand_access_policies,
                page_size=page_size,
            )
        )

    def assign_access_policy_to_user(
        self, user_id: UserId, access_policy_ids: list[AccessPolicyId]
    ) -> None:
        """3.6 Assign Access Policy to User"""
        body = {"access_policy_ids": access_policy_ids}

        r = self._session.put(
            f"{self._base_url}/users/{user_id}/access_policies", json=body
        )
        return Response[None].validate_and_unwrap(r)

    def assign_nfc_card_to_user(
        self, user_id: UserId, token: NfcCardToken, force_add: bool | None = None
    ) -> None:
        """3.7 Assign NFC Card to User"""

        body = AssignNfcCardRequest(token=token, force_add=force_add).model_dump(
            exclude_none=True
        )

        r = self._session.put(f"{self._base_url}/users/{user_id}/nfc_cards", json=body)
        return Response[None].validate_and_unwrap(r)

    def unassign_nfc_card_from_user(self, user_id: UserId, token: NfcCardToken) -> None:
        """3.8 Unassign NFC Card from User"""
        body = {"token": token}

        r = self._session.put(
            f"{self._base_url}/users/{user_id}/nfc_cards/delete", json=body
        )
        return Response[None].validate_and_unwrap(r)

    def assign_pin_code_to_user(self, user_id: UserId, pin_code: str) -> None:
        """3.9 Assign PIN Code to User"""
        body = {"pin_code": pin_code}

        r = self._session.put(f"{self._base_url}/users/{user_id}/pin_codes", json=body)
        return Response[None].validate_and_unwrap(r)

    def unassign_pin_code_from_user(self, user_id: UserId) -> None:
        """3.10 Unassign PIN Code from User"""

        r = self._session.delete(f"{self._base_url}/users/{user_id}/pin_codes")
        return Response[None].validate_and_unwrap(r)

    def create_user_group(
        self,
        name: str,
        up_id: UserGroupId | None = None,
    ) -> UserGroup | None:
        """3.11 Create User Group
        !!! note "The docs do not describe this as returning any data"
            Returns None if the user group already existed
        """

        class CreateUserGroupRequest(ForbidExtraBaseModel):
            name: str
            up_id: UserGroupId | None = None

        body = CreateUserGroupRequest(name=name, up_id=up_id).model_dump(
            exclude_none=True
        )

        r = self._session.post(f"{self._base_url}/user_groups", json=body)
        return Response[UserGroup | None].validate_and_unwrap(r)

    def fetch_all_user_groups(self) -> list[UserGroup]:
        """3.12 Fetch All User Groupes"""

        r = self._session.get(f"{self._base_url}/user_groups")
        return Response[list[UserGroup]].validate_and_unwrap(r)

    def fetch_user_group(
        self,
        user_group_id: UserGroupId,
    ) -> UserGroup:
        """3.13 Fetch User Group"""

        r = self._session.get(f"{self._base_url}/user_groups/{user_group_id}")
        return Response[UserGroup].validate_and_unwrap(r)

    def update_user_group(
        self,
        user_group_id: UserGroupId,
        name: str,
        up_id: UserGroupId | None = None,
    ) -> None:
        """3.14 Update User Group"""

        class UpdateUserGroupRequest(ForbidExtraBaseModel):
            name: str
            up_id: UserGroupId | None = None

        body = UpdateUserGroupRequest(name=name, up_id=up_id).model_dump(
            exclude_none=True
        )

        r = self._session.put(
            f"{self._base_url}/user_groups/{user_group_id}", json=body
        )
        return Response[None].validate_and_unwrap(r)

    def delete_user_group(
        self,
        user_group_id: UserGroupId,
    ) -> None:
        """3.15 Delete User Group"""

        r = self._session.delete(f"{self._base_url}/user_groups/{user_group_id}")
        return Response[None].validate_and_unwrap(r)

    def assign_user_to_user_group(
        self,
        user_group_id: UserGroupId,
        users: list[UserId],
    ) -> None:
        """3.16 Assign User to User Group"""

        r = self._session.post(
            f"{self._base_url}/user_groups/{user_group_id}/users", json=users
        )
        return Response[None].validate_and_unwrap(r)

    def unassign_user_from_user_group(
        self,
        user_group_id: UserGroupId,
        users: list[UserId],
    ) -> None:
        """3.17 Unassign User from User Group"""

        r = self._session.post(
            f"{self._base_url}/user_groups/{user_group_id}/users/delete", json=users
        )
        return Response[None].validate_and_unwrap(r)

    def fetch_users_in_a_user_group(
        self,
        user_group_id: UserGroupId,
    ) -> list[User]:
        """3.18 Fetch Users in a User Group"""

        r = self._session.get(f"{self._base_url}/user_groups/{user_group_id}/users")
        return Response[list[User]].validate_and_unwrap(r)

    def fetch_all_users_in_a_user_group(self, user_group_id: UserGroupId) -> list[User]:
        """3.19 Fetch All Users in a User Group"""

        r = self._session.get(f"{self._base_url}/user_groups/{user_group_id}/users/all")
        return Response[list[User]].validate_and_unwrap(r)

    def fetch_the_access_policies_assigned_to_a_user(
        self, user_id: UserId, only_user_policies: bool | None = None
    ) -> list[AccessPolicy]:
        """3.20 Fetch the Access Policies Assigned to a User"""

        params = {"only_user_policies": "true" if only_user_policies else "false"}
        r = self._session.get(
            f"{self._base_url}/users/{user_id}/access_policies", params=params
        )
        return Response[list[AccessPolicy]].validate_and_unwrap(r)

    def assign_access_policy_to_user_group(
        self, user_group_id: UserGroupId, access_policy_ids: list[AccessPolicyId]
    ) -> None:
        """3.21 Assign Access Policy to User Group"""

        body = {"access_policy_ids": access_policy_ids}

        r = self._session.put(
            f"{self._base_url}/user_groups/{user_group_id}/access_policies", json=body
        )
        return Response[None].validate_and_unwrap(r)

    def fetch_the_access_policies_assigned_to_a_user_group(
        self,
        user_group_id: UserGroupId,
    ) -> list[AccessPolicy]:
        """3.22 Fetch the Access Policies Assigned to a User Group"""

        r = self._session.get(
            f"{self._base_url}/user_groups/{user_group_id}/access_policies"
        )
        return Response[list[AccessPolicy]].validate_and_unwrap(r)

    def create_visitor(  # noqa: PLR0913
        self,
        first_name: str,
        last_name: str,
        start_time: UnixTimestampDateTime,
        end_time: UnixTimestampDateTime,
        visit_reason: VisitReason,
        remarks: str | None = None,
        mobile_phone: str | None = None,
        email: str | None = None,
        visitor_company: str | None = None,
        resources: Sequence[Resource] | None = None,
        week_schedule: WeekSchedule | None = None,
    ) -> Visitor:
        """4.2 Create Visitor"""

        class CreateVisitorRequest(ForbidExtraBaseModel):
            first_name: str
            """First name of the visitor."""
            last_name: str
            """Last name of the visitor."""
            remarks: str | None = None
            """Remarks about the visitor."""
            mobile_phone: str | None = None
            """Mobile phone number of the visitor."""
            email: str | None = None
            """Email address of the visitor."""
            visitor_company: str | None = None
            """Company name of the visitor."""
            start_time: UnixTimestampDateTime
            """Start time of the visit."""
            end_time: UnixTimestampDateTime
            """End time of the visit."""
            visit_reason: VisitReason
            resources: Sequence[Resource] | None = None
            """Assign available locations to the visitor."""
            week_schedule: WeekSchedule | None = None
            """The customizable scheduling strategy for each day from
            Sunday to Saturday. If not specified, it means access is
            allowed every day."""

        body = CreateVisitorRequest(
            first_name=first_name,
            last_name=last_name,
            start_time=start_time,
            end_time=end_time,
            visit_reason=visit_reason,
            remarks=remarks,
            mobile_phone=mobile_phone,
            email=email,
            visitor_company=visitor_company,
            resources=resources,
            week_schedule=week_schedule,
        ).model_dump(exclude_none=True)

        r = self._session.post(f"{self._base_url}/visitors", json=body)
        return Response[Visitor].validate_and_unwrap(r)

    def fetch_visitor(
        self,
        visitor_id: VisitorId,
    ) -> Visitor:
        """4.3 Fetch Visitor"""

        r = self._session.get(f"{self._base_url}/visitors/{visitor_id}")
        return Response[Visitor].validate_and_unwrap(r)

    def fetch_all_visitors(
        self,
        status: VisitorStatus | None = None,
        keyword: str | None = None,
        expand: list[FetchAllVisitorsExpansion] | None = None,
        page_num: int | None = None,
        page_size: int | None = None,
    ) -> SuccessResponse[list[Visitor], ResponsePagination]:
        """4.4 Fetch All Visitors

        If you don't need to manually handle pagination, consider [`fetch_all_visitors__unpaged`][unifi_access.AccessClient.fetch_all_visitors__unpaged].
        """

        class FetchAllVisitorsRequest(RequestPagination):
            model_config = ConfigDict(populate_by_name=True)

            status: VisitorStatus | None = None
            """The visitor's status. enum status"""
            keyword: str | None = None
            """Support prefix matching for first name and last
            name. NOTE: The status filtering is disabled when
            searching with keyword."""
            expand: list[FetchAllVisitorsExpansion] | None = Field(
                alias=str("expand[]"),  # noqa: UP018
                default=None,
            )
            """Determine whether to return the objects (Optional). For
            the "none" option, it means that no object will be
            returned."""

        params = FetchAllVisitorsRequest(
            status=status,
            keyword=keyword,
            expand=expand,
            page_num=page_num,
            page_size=page_size,
        ).model_dump(exclude_none=True, by_alias=True)

        r = self._session.get(f"{self._base_url}/visitors", params=params)
        return (
            PaginatedResponse[list[Visitor]].model_validate_json(r.content).root
        ).success_or_raise()

    def fetch_all_visitors__unpaged(
        self,
        status: VisitorStatus | None = None,
        keyword: str | None = None,
        expand: list[FetchAllVisitorsExpansion] | None = None,
        page_size: int | None = None,
    ) -> Iterable[Visitor]:
        """4.4 Fetch All Visitors

        This will automatically handle pagination.
        If you need more control, consider [`fetch_all_visitors`][unifi_access.AccessClient.fetch_all_visitors].
        """
        yield from iterate_pages(
            functools.partial(
                self.fetch_all_visitors,
                status=status,
                keyword=keyword,
                expand=expand,
                page_size=page_size,
            )
        )

    def update_visitor(  # noqa: PLR0913
        self,
        visitor_id: VisitorId,
        first_name: str | None = None,
        last_name: str | None = None,
        start_time: UnixTimestampDateTime | None = None,
        end_time: UnixTimestampDateTime | None = None,
        visit_reason: VisitReason | None = None,
        remarks: str | None = None,
        mobile_phone: str | None = None,
        email: str | None = None,
        visitor_company: str | None = None,
        resources: Sequence[Resource] | None = None,
        week_schedule: WeekSchedule | None = None,
    ) -> Visitor:
        """4.5 Update Visitor"""

        class UpdateVisitorRequest(ForbidExtraBaseModel):
            first_name: str | None = None
            """First name of the visitor."""
            last_name: str | None = None
            """Last name of the visitor."""
            remarks: str | None = None
            """Remarks about the visitor."""
            mobile_phone: str | None = None
            """Mobile phone number of the visitor."""
            email: str | None = None
            """Email address of the visitor."""
            visitor_company: str | None = None
            """Company name of the visitor."""
            start_time: UnixTimestampDateTime | None = None
            """Start time of the visit."""
            end_time: UnixTimestampDateTime | None = None
            """End time of the visit."""
            visit_reason: VisitReason | None = None
            """Assign available locations to the visitor."""
            resources: Sequence[Resource] | None = None
            """The customizable scheduling strategy for each day from
            Sunday to Saturday. If not specified, it means access is
            allowed every day."""
            week_schedule: WeekSchedule | None = None

        body = UpdateVisitorRequest(
            first_name=first_name,
            last_name=last_name,
            start_time=start_time,
            end_time=end_time,
            visit_reason=visit_reason,
            remarks=remarks,
            mobile_phone=mobile_phone,
            email=email,
            visitor_company=visitor_company,
            resources=resources,
            week_schedule=week_schedule,
        ).model_dump(exclude_none=True)

        r = self._session.put(f"{self._base_url}/visitors/{visitor_id}", json=body)
        return Response[Visitor].validate_and_unwrap(r)

    def delete_visitor(
        self, visitor_id: VisitorId, is_force: bool | None = None
    ) -> None:
        """4.6 Delete Visitor"""

        params = {"is_force": "true" if is_force else "false"}
        r = self._session.delete(
            f"{self._base_url}/visitors/{visitor_id}", params=params
        )
        return Response[None].validate_and_unwrap(r)

    def assign_nfc_card_to_visitor(
        self, visitor_id: VisitorId, token: NfcCardToken, force_add: bool | None = None
    ) -> None:
        """4.7 Assign NFC Card to Visitor"""
        body = AssignNfcCardRequest(token=token, force_add=force_add).model_dump(
            exclude_none=True
        )

        r = self._session.put(
            f"{self._base_url}/visitors/{visitor_id}/nfc_cards", json=body
        )
        return Response[None].validate_and_unwrap(r)

    def unassign_nfc_card_from_visitor(
        self, visitor_id: VisitorId, token: NfcCardToken
    ) -> None:
        """4.8 Unassign NFC Card from Visitor"""
        body = {"token": token}

        r = self._session.put(
            f"{self._base_url}/visitors/{visitor_id}/nfc_cards/delete", json=body
        )
        return Response[None].validate_and_unwrap(r)

    def assign_pin_code_to_visitor(self, visitor_id: VisitorId, pin_code: str) -> None:
        """4.9 Assign PIN Code to Visitor"""
        body = {"pin_code": pin_code}

        r = self._session.put(
            f"{self._base_url}/visitors/{visitor_id}/pin_codes", json=body
        )
        return Response[None].validate_and_unwrap(r)

    def unassign_pin_code_from_visitor(self, visitor_id: VisitorId) -> None:
        """4.10 Unassign PIN Code from Visitor"""

        r = self._session.delete(f"{self._base_url}/visitors/{visitor_id}/pin_codes")
        return Response[None].validate_and_unwrap(r)

    def create_access_policy(
        self,
        name: str,
        schedule_id: ScheduleId,
        resources: Sequence[Resource] | None = None,
    ) -> AccessPolicy:
        """5.2 Create Access Policy"""

        class CreateAccessPolicyRequest(ForbidExtraBaseModel):
            name: str
            resource: Sequence[Resource] | None = None
            schedule_id: ScheduleId

        body = CreateAccessPolicyRequest(
            name=name, schedule_id=schedule_id, resource=resources
        ).model_dump(exclude_none=True)

        r = self._session.post(f"{self._base_url}/access_policies", json=body)
        return Response[AccessPolicy].validate_and_unwrap(r)

    def update_access_policy(
        self,
        access_policy_id: AccessPolicyId,
        name: str | None = None,
        resources: Sequence[Resource] | None = None,
        schedule_id: ScheduleId | None = None,
    ) -> AccessPolicy:
        """5.3 Update Access Policy"""

        class UpdateAccessPolicyRequest(ForbidExtraBaseModel):
            name: str | None = None
            resource: Sequence[Resource] | None = None
            schedule_id: ScheduleId | None = None

        body = UpdateAccessPolicyRequest(
            name=name, schedule_id=schedule_id, resource=resources
        ).model_dump(exclude_none=True)

        r = self._session.put(
            f"{self._base_url}/access_policies/{access_policy_id}", json=body
        )
        return Response[AccessPolicy].validate_and_unwrap(r)

    def delete_access_policy(
        self, access_policy_id: AccessPolicyId
    ) -> Literal["success"]:
        """5.4 Delete Access Policy"""

        r = self._session.delete(f"{self._base_url}/access_policies/{access_policy_id}")
        return Response[Literal["success"]].validate_and_unwrap(r)

    def fetch_access_policy(self, access_policy_id: AccessPolicyId) -> AccessPolicy:
        """5.5 Fetch Access Policy"""

        r = self._session.get(f"{self._base_url}/access_policies/{access_policy_id}")
        return Response[AccessPolicy].validate_and_unwrap(r)

    def fetch_all_access_policies(self) -> list[AccessPolicy]:
        """5.6 Fetch All Access Policies"""

        r = self._session.get(f"{self._base_url}/access_policies")
        return Response[list[AccessPolicy]].validate_and_unwrap(r)

    def create_holiday_group(
        self,
        name: str,
        description: str | None = None,
        holidays: list[PartialHoliday] | None = None,
    ) -> HolidayGroup:
        """5.8 Create Holiday Group"""

        class CreateHolidayGroupRequest(ForbidExtraBaseModel):
            name: str
            description: str | None = None
            holidays: list[PartialHoliday] | None = None

        body = CreateHolidayGroupRequest(
            name=name, description=description, holidays=holidays
        ).model_dump(exclude_none=True)

        r = self._session.post(
            f"{self._base_url}/access_policies/holiday_groups", json=body
        )
        return Response[HolidayGroup].validate_and_unwrap(r)

    def update_holiday_group(
        self,
        holiday_group_id: HolidayGroupId,
        # NOTE: the docs mark `name` as required
        name: str | None = None,
        description: str | None = None,
        holidays: list[PartialHoliday] | None = None,
    ) -> HolidayGroup:
        """5.9 Update Holiday Group"""

        class UpdateHolidayGroupRequest(ForbidExtraBaseModel):
            name: str | None = None
            description: str | None = None
            holidays: list[PartialHoliday] | None = None

        body = UpdateHolidayGroupRequest(
            name=name, description=description, holidays=holidays
        ).model_dump(exclude_none=True)

        r = self._session.put(
            f"{self._base_url}/access_policies/holiday_groups/{holiday_group_id}",
            json=body,
        )
        return Response[HolidayGroup].validate_and_unwrap(r)

    def delete_holiday_group(
        self, holiday_group_id: HolidayGroupId
    ) -> Literal["success"]:
        """5.10 Delete Holiday Group"""

        r = self._session.delete(
            f"{self._base_url}/access_policies/holiday_groups/{holiday_group_id}"
        )
        return Response[Literal["success"]].validate_and_unwrap(r)

    def fetch_holiday_group(self, holiday_group_id: HolidayGroupId) -> HolidayGroup:
        """5.11 Fetch Holiday Group"""

        r = self._session.get(
            f"{self._base_url}/access_policies/holiday_groups/{holiday_group_id}"
        )
        return Response[HolidayGroup].validate_and_unwrap(r)

    def fetch_all_holiday_groups(self) -> list[FetchAllHolidayGroupsResponse]:
        """5.12 Fetch All Holiday Groups"""

        r = self._session.get(f"{self._base_url}/access_policies/holiday_groups")
        return Response[list[FetchAllHolidayGroupsResponse]].validate_and_unwrap(r)

    def create_schedule(
        self,
        name: str,
        # XXX: figure out what if "not specified" means
        week_schedule: WeekSchedule,
        holiday_group_id: HolidayGroupId | None = None,
        holiday_schedule: list[TimePeriod] | None = None,
    ) -> Schedule:
        """5.14 Create Schedule"""

        class CreateScheduleRequest(ForbidExtraBaseModel):
            name: str
            week_schedule: WeekSchedule
            holiday_group_id: HolidayGroupId | None = None
            holiday_schedule: list[TimePeriod] | None = None

        body = CreateScheduleRequest(
            name=name,
            week_schedule=week_schedule,
            holiday_group_id=holiday_group_id,
            holiday_schedule=holiday_schedule,
        ).model_dump(exclude_none=True)

        r = self._session.post(f"{self._base_url}/access_policies/schedules", json=body)
        return Response[Schedule].validate_and_unwrap(r)

    def update_schedule(
        self,
        schedule_id: ScheduleId,
        name: str | None = None,
        # XXX: figure out what "if not specified" means. Do I need to
        #      differentiate between None and missing here?
        week_schedule: WeekSchedule | None = None,
        holiday_group_id: HolidayGroupId | None = None,
        holiday_schedule: list[TimePeriod] | None = None,
    ) -> dict[Any, Any]:  # TODO: this should really return dict[Never, Never]
        """5.15 Update Schedule"""

        class UpdateScheduleRequest(ForbidExtraBaseModel):
            name: str | None = None
            week_schedule: WeekSchedule | None = None
            holiday_group_id: HolidayGroupId | None = None
            holiday_schedule: list[TimePeriod] | None = None

        body = UpdateScheduleRequest(
            name=name,
            week_schedule=week_schedule,
            holiday_group_id=holiday_group_id,
            holiday_schedule=holiday_schedule,
        ).model_dump(exclude_none=True)

        r = self._session.put(
            f"{self._base_url}/access_policies/schedules/{schedule_id}", json=body
        )
        return Response[dict[Any, Any]].validate_and_unwrap(r)

    # NOTE: the API docs incorrectly define the response as a subset of `Schedule`
    def fetch_schedule(self, schedule_id: ScheduleId) -> Schedule:
        """5.16 Fetch Schedule"""

        r = self._session.get(
            f"{self._base_url}/access_policies/schedules/{schedule_id}"
        )
        return Response[Schedule].validate_and_unwrap(r)

    def fetch_all_schedules(self) -> list[FetchAllSchedulesResponse]:
        """5.17 Fetch All Schedules"""

        r = self._session.get(f"{self._base_url}/access_policies/schedules")
        return Response[list[FetchAllSchedulesResponse]].validate_and_unwrap(r)

    def delete_schedule(self, schedule_id: ScheduleId) -> Literal["success"]:
        """5.18 Delete Schedule"""

        r = self._session.delete(
            f"{self._base_url}/access_policies/schedules/{schedule_id}"
        )
        return Response[Literal["success"]].validate_and_unwrap(r)

    def generate_pin_code(self) -> str:
        r = self._session.post(f"{self._base_url}/credentials/pin_codes")
        return Response["str"].validate_and_unwrap(r)

    def begin_enroll_card(
        self, device_id: DeviceId, reset_ua_card: bool | None = None
    ) -> EnrollNfcCardResponse:
        """6.2 Enroll NFC Card"""

        class EnrollNFCCardRequest(ForbidExtraBaseModel):
            device_id: DeviceId
            reset_ua_card: bool | None = None

        body = EnrollNFCCardRequest(
            device_id=device_id, reset_ua_card=reset_ua_card
        ).model_dump(exclude_none=True)

        r = self._session.post(
            f"{self._base_url}/credentials/nfc_cards/sessions", json=body
        )
        return Response[EnrollNfcCardResponse].validate_and_unwrap(r)

    def fetch_enroll_card_status(
        self,
        session_id: NfcCardEnrollmentSessionId,
    ) -> NfcCardEnrollmentStatus:
        """6.3 Fetch NFC Card Enrollment Status"""
        r = self._session.get(
            f"{self._base_url}/credentials/nfc_cards/sessions/{session_id}"
        )
        return Response[NfcCardEnrollmentStatus].validate_and_unwrap(r)

    def remove_enrollment_session(
        self,
        session_id: NfcCardEnrollmentSessionId,
    ) -> Literal["success"]:
        """6.4 Remove a Session Created for NFC Card Enrollment"""
        r = self._session.delete(
            f"{self._base_url}/credentials/nfc_cards/sessions/{session_id}"
        )
        return Response[Literal["success"]].validate_and_unwrap(r)

    def fetch_nfc_card(self, nfc_card_token: NfcCardToken) -> NfcCard:
        """6.7 Fetch NFC Card"""
        r = self._session.get(
            f"{self._base_url}/credentials/nfc_cards/tokens/{nfc_card_token}"
        )
        return Response[NfcCard].validate_and_unwrap(r)

    def fetch_all_nfc_cards(
        self, page_num: int | None = None, page_size: int | None = None
    ) -> SuccessResponse[list[NfcCard], ResponsePagination]:
        """6.8 Fetch NFC Cards

        If you don't need to manually handle pagination, consider [`fetch_all_nfc_cards__unpaged`][unifi_access.AccessClient.fetch_all_nfc_cards__unpaged].
        """
        params = RequestPagination(page_num=page_num, page_size=page_size)
        r = self._session.get(
            f"{self._base_url}/credentials/nfc_cards/tokens", params=params
        )
        return (
            PaginatedResponse[list[NfcCard]].model_validate_json(r.content).root
        ).success_or_raise()

    def fetch_all_nfc_cards__unpaged(
        self, page_size: int | None = None
    ) -> Iterable[NfcCard]:
        """6.8 Fetch NFC Cards

        This will automatically handle pagination.
        If you need more control, consider [`fetch_all_nfc_cards`][unifi_access.AccessClient.fetch_all_nfc_cards].
        """
        yield from iterate_pages(
            functools.partial(self.fetch_all_nfc_cards, page_size=page_size)
        )

    def delete_nfc_card(self, nfc_card_token: NfcCardToken) -> Literal["success"]:
        """6.7 Fetch NFC Card"""
        r = self._session.delete(
            f"{self._base_url}/credentials/nfc_cards/tokens/{nfc_card_token}"
        )
        return Response[Literal["success"]].validate_and_unwrap(r)

    def fetch_door_group_topology(self) -> list[DoorGroupTopology]:
        """7.1 Fetch Door Group Topology"""
        r = self._session.get(f"{self._base_url}/door_groups/topology")
        return Response[list[DoorGroupTopology]].validate_and_unwrap(r)

    def create_door_group(
        self, group_name: str, resources: list[ResourceId]
    ) -> DoorGroup[Resource]:
        """7.2 Create Door Group"""
        body = {"group_name": group_name, "resources": resources}

        r = self._session.post(f"{self._base_url}/door_groups", json=body)
        return Response[DoorGroup[Resource]].validate_and_unwrap(r)

    def fetch_door_group(self, door_group_id: DoorGroupId) -> DoorGroup[NamedResource]:
        """7.3 Fetch Door Group"""
        r = self._session.get(f"{self._base_url}/door_groups/{door_group_id}")
        return Response[DoorGroup[NamedResource]].validate_and_unwrap(r)

    def update_door_group(
        self,
        door_group_id: DoorGroupId,
        group_name: str | None = None,
        resources: list[ResourceId] | None = None,
    ) -> DoorGroup[Resource]:
        """7.4 Update Door Group"""

        class UpdateDoorGroupRequest(ForbidExtraBaseModel):
            group_name: str | None = None
            resources: list[ResourceId] | None = None

        body = UpdateDoorGroupRequest(
            group_name=group_name, resources=resources
        ).model_dump(exclude_none=True)

        r = self._session.put(
            f"{self._base_url}/door_groups/{door_group_id}", json=body
        )
        return Response[DoorGroup[Resource]].validate_and_unwrap(r)

    def fetch_all_door_groups(self) -> list[DoorGroup[Resource]]:
        """7.5 Fetch All Door Groups"""
        r = self._session.get(f"{self._base_url}/door_groups")
        return Response[list[DoorGroup[Resource]]].validate_and_unwrap(r)

    def delete_door_group(self, door_group_id: DoorGroupId) -> Literal["success"]:
        """7.6 Delete Door Group"""
        r = self._session.get(f"{self._base_url}/door_groups/{door_group_id}")
        return Response[Literal["success"]].validate_and_unwrap(r)

    def fetch_door(self, door_id: DoorId) -> Door:
        """7.7 Fetch Door"""
        r = self._session.get(f"{self._base_url}/doors/{door_id}")
        return Response[Door].validate_and_unwrap(r)

    def fetch_all_doors(self) -> list[Door]:
        """7.8 Fetch All Doors"""
        r = self._session.get(f"{self._base_url}/doors")
        return Response[list[Door]].validate_and_unwrap(r)

    def unlock_door(self, door_id: DoorId) -> Literal["success"]:
        """7.9 Remote Door Unlocking"""
        r = self._session.put(f"{self._base_url}/doors/{door_id}/unlock")
        return Response[Literal["success"]].validate_and_unwrap(r)

    # XXX: could do a better job of typing for interval
    def set_temporary_door_locking_rule(
        self, door_id: DoorId, type_: DoorLockingRuleType, interval: int | None = None
    ) -> Literal["success"]:
        """7.10 Set Temporary Door Locking Rule"""

        class SetTemporaryDoorLockingRuleRequest(ForbidExtraBaseModel):
            type: DoorLockingRuleType
            interval: int | None = None

        body = SetTemporaryDoorLockingRuleRequest(
            type=type_, interval=interval
        ).model_dump(exclude_none=True)
        r = self._session.put(f"{self._base_url}/doors/{door_id}/lock_rule", json=body)
        return Response[Literal["success"]].validate_and_unwrap(r)

    def fetch_door_locking_rule(self, door_id: DoorId) -> DoorLockingRule:
        """7.11 Fetch Door Locking Rule"""
        r = self._session.get(f"{self._base_url}/doors/{door_id}/lock_rule")
        return Response[DoorLockingRule].validate_and_unwrap(r)

    # TODO: why is this two bools instead of an enum? Could probably be improved.
    def set_door_emergency_status(
        self,
        lockdown: bool = False,
        evacuation: bool = False,
    ) -> Literal["success"]:
        """7.12 Set Door Emergency Status"""

        body = DoorEmergencyStatus(lockdown=lockdown, evacuation=evacuation).model_dump(
            exclude_none=True
        )
        r = self._session.put(f"{self._base_url}/doors/settings/emergency", json=body)
        return Response[Literal["success"]].validate_and_unwrap(r)

    def fetch_door_emergency_status(self) -> DoorEmergencyStatus:
        """7.13 Fetch Door Emergency Status"""
        r = self._session.get(f"{self._base_url}/doors/settings/emergency")
        return Response[DoorEmergencyStatus].validate_and_unwrap(r)

    def fetch_devices(self) -> list[Device]:
        """8.1 Fetch Devices"""
        r = self._session.get(f"{self._base_url}/devices")
        return Response[list[Device]].validate_and_unwrap(r)

    def fetch_system_logs(  # noqa: PLR0913
        self,
        topic: SystemLogTopic,
        since: datetime.datetime | None = None,
        until: datetime.datetime | None = None,
        actor_id: ActorId | None = None,
        page_num: int | None = None,
        page_size: int | None = None,
    ) -> SuccessResponse[FetchSystemLogsResponse, ResponsePagination]:
        """9.2 Fetch System Logs

        If you don't need to manually handle pagination, consider [`fetch_all_system_logs__unpaged`][unifi_access.AccessClient.fetch_all_system_logs__unpaged].
        """
        params = RequestPagination(page_num=page_num, page_size=page_size).model_dump(
            exclude_none=True
        )

        class FetchSystemLogsRequest(ForbidExtraBaseModel):
            topic: SystemLogTopic
            since: UnixTimestampDateTime | None = None
            until: UnixTimestampDateTime | None = None
            actor_id: ActorId | None = None

        body = FetchSystemLogsRequest(
            topic=topic,
            since=since,
            until=until,
            actor_id=actor_id,
        ).model_dump(exclude_none=True)

        r = self._session.post(
            f"{self._base_url}/system/logs", params=params, json=body
        )
        return (
            PaginatedResponse[FetchSystemLogsResponse].model_validate_json(r.content)
        ).root.success_or_raise()

    def fetch_system_logs__unpaged(
        self,
        topic: SystemLogTopic,
        since: datetime.datetime | None = None,
        until: datetime.datetime | None = None,
        actor_id: ActorId | None = None,
        page_size: int | None = None,
    ) -> Iterable[SystemLogEntry]:
        """9.2 Fetch System Logs

        This will automatically handle pagination.
        If you need more control, consider [`fetch_all_system_logs`][unifi_access.AccessClient.fetch_all_system_logs].
        """

        # can't just use `functools.partial` here, because we need to have `.data` return a list
        # instead of `FetchSystemLogsResponse`
        def extract_hits_wrapper(
            page_num: int,
        ) -> SuccessResponse[list[SystemLogEntry], ResponsePagination]:
            resp = self.fetch_system_logs(
                topic=topic,
                since=since,
                until=until,
                actor_id=actor_id,
                page_size=page_size,
                page_num=page_num,
            )

            return SuccessResponse[list[SystemLogEntry], ResponsePagination](
                code=resp.code,
                msg=resp.msg,
                data=resp.data.hits,
                pagination=resp.pagination,
            )

        yield from iterate_pages(extract_hits_wrapper)

    def export_system_logs(
        self,
        topic: SystemLogTopic,
        since: datetime.datetime,
        until: datetime.datetime,
        timezone: str,
        actor_id: ActorId | None = None,
    ) -> bytes:
        """9.3 Export System Logs"""

        class ExportSystemLogsRequest(ForbidExtraBaseModel):
            topic: SystemLogTopic
            since: UnixTimestampDateTime
            until: UnixTimestampDateTime
            timezone: str
            actor_id: ActorId | None = None

        body = ExportSystemLogsRequest(
            topic=topic,
            since=since,
            until=until,
            timezone=timezone,
            actor_id=actor_id,
        ).model_dump(exclude_none=True)

        r = self._session.post(f"{self._base_url}/system/logs/export", json=body)
        return r.content

    def send_unifi_identity_invitations(
        self, users: list[IdentityInvitationUser]
    ) -> list[IdentityInvitationEmailFailure]:
        """10.1 Send UniFi Identity Invitations"""
        body = TypeAdapter(list[IdentityInvitationUser]).dump_python(
            users, exclude_none=True
        )

        r = self._session.post(
            f"{self._base_url}/users/identity/invitations", json=body
        )
        return Response[list[IdentityInvitationEmailFailure]].validate_and_unwrap(r)

    def fetch_available_resources(
        self, resource_type: Iterable[IdentityResourceType] | None = None
    ) -> dict[IdentityResourceType, list[IdentityResource]]:
        """10.2 Fetch Available Resources"""

        params = {"resource_type": ",".join(resource_type)} if resource_type else None

        r = self._session.get(
            f"{self._base_url}/users/identity/assignments", params=params
        )
        return Response[
            dict[IdentityResourceType, list[IdentityResource]]
        ].validate_and_unwrap(r)

    def assign_resources_to_user(
        self,
        user_id: UserId,
        resource_type: IdentityResourceType,
        resource_ids: list[IdentityResourceId],
    ) -> None:
        """10.3 Assign Resources to Users"""
        body = {"resource_type": resource_type, "resource_ids": resource_ids}

        r = self._session.post(
            f"{self._base_url}/users/{user_id}/identity/assignments", json=body
        )
        return Response[None].validate_and_unwrap(r)

    def fetch_resources_assigned_to_user(
        self,
        user_id: UserId,
    ) -> dict[IdentityResourceType, list[IdentityResource]]:
        """10.4 Fetch Resources Assigned to Users"""

        r = self._session.get(f"{self._base_url}/users/{user_id}/identity/assignments")
        return Response[
            dict[IdentityResourceType, list[IdentityResource]]
        ].validate_and_unwrap(r)

    def assign_resources_to_user_group(
        self,
        user_group_id: UserGroupId,
        resource_type: IdentityResourceType,
        resource_ids: list[IdentityResourceId],
    ) -> None:
        """10.5 Assign Resources to User Groups"""
        body = {"resource_type": resource_type, "resource_ids": resource_ids}

        r = self._session.post(
            f"{self._base_url}/user_groups/{user_group_id}/identity/assignments",
            json=body,
        )
        return Response[None].validate_and_unwrap(r)

    def fetch_resources_assigned_to_user_group(
        self,
        user_group_id: UserGroupId,
    ) -> dict[IdentityResourceType, list[IdentityResource]]:
        """10.6 Fetch Resources Assigned to User Groups"""

        r = self._session.get(
            f"{self._base_url}/user_groups/{user_group_id}/identity/assignments"
        )
        return Response[
            dict[IdentityResourceType, list[IdentityResource]]
        ].validate_and_unwrap(r)
