from ._base import (
    AccessPolicyId,
    ForbidExtraBaseModel,
    FormattedTime,
    HolidayGroupId,
    HolidayId,
    Resource,
    RFC3339Datetime,
    ScheduleId,
)


class AccessPolicy(ForbidExtraBaseModel):
    """5.1 Access Policy Schemas"""

    id: AccessPolicyId
    """Identity ID of the access policy."""
    name: str
    """Name of the access policy."""
    resources: list[Resource]
    """Specify the locations that can be accessed."""
    schedule_id: ScheduleId
    """Identity ID of the schedule."""


class Holiday(ForbidExtraBaseModel):
    description: str | None = None
    """Description of the holiday."""
    id: HolidayId
    """Identity ID of the holiday."""
    name: str
    """Name of the holiday."""
    repeat: bool
    """Indicate whether the holiday repeats annually."""
    start_time: RFC3339Datetime
    """Start time of the holiday"""
    end_time: RFC3339Datetime
    """End time of the holiday"""
    is_template: bool | None = None
    """!!! note "not listed in API docs" """


class PartialHoliday(ForbidExtraBaseModel):
    description: str | None = None
    """Description of the holiday."""
    id: HolidayId | None = None
    """Identity ID of the holiday."""
    name: str | None = None
    """Name of the holiday."""
    repeat: bool | None = None
    """Indicate whether the holiday repeats annually."""
    start_time: RFC3339Datetime | None = None
    """Start time of the holiday"""
    end_time: RFC3339Datetime | None = None
    """End time of the holiday"""


class HolidayGroup(ForbidExtraBaseModel):
    id: HolidayGroupId
    """Identity ID of the holiday group."""
    name: str
    """Name of the holiday group."""
    is_default: bool
    """Indicate whether the holiday group is the system default."""
    description: str
    """Description of the holiday group."""
    holidays: list[Holiday] | None = None
    """A list of the holidays within the holiday group."""
    template_name: str | None = None
    """!!! note "not listed in API docs" """
    is_private: bool | None = None
    """!!! note "not listed in API docs" """


class FetchAllHolidayGroupsResponse(ForbidExtraBaseModel):
    id: HolidayGroupId
    """Identity ID of the holiday group."""
    name: str
    """Name of the holiday group."""
    is_default: bool
    """
    Indicate whether the holiday group is the system default.
    !!! note "not listed in API docs for this call"
    """
    description: str
    """Description of the holiday group."""
    count: int
    """Total number of holidays"""
    is_private: bool | None = None
    """!!! note "not listed in API docs" """


class TimePeriod(ForbidExtraBaseModel):
    start_time: FormattedTime
    """Start time of the access time period."""
    end_time: FormattedTime
    """End time of the access time period."""


class WeekSchedule(ForbidExtraBaseModel):
    sunday: list[TimePeriod]
    monday: list[TimePeriod]
    tuesday: list[TimePeriod]
    wednesday: list[TimePeriod]
    thursday: list[TimePeriod]
    friday: list[TimePeriod]
    saturday: list[TimePeriod]


class _ScheduleBase(ForbidExtraBaseModel):
    id: ScheduleId
    """Identity ID of the schedule."""
    name: str
    """Name of the schedule."""
    is_default: bool
    """Indicate whether the schedule is the system default."""
    type: str
    """Contains the access type, which is assigned to a user along with an access policy."""


class Schedule(_ScheduleBase):
    weekly: WeekSchedule
    """
    The customizable scheduling strategy for each day from Sunday to Saturday.
    If not specified, it means access is allowed every day.
    """
    holiday_schedule: list[TimePeriod]
    """Specify the accessible period during holidays. UniFi Access Requirement: 1.20.11 or later"""
    holiday_group_id: HolidayGroupId
    """Identity ID of the holiday group."""
    holiday_group: HolidayGroup


class PartialSchedule(_ScheduleBase):
    weekly: WeekSchedule | None = None
    """
    The customizable scheduling strategy for each day from Sunday to Saturday.
    If not specified, it means access is allowed every day.
    """
    holiday_schedule: list[TimePeriod] | None = None
    """Specify the accessible period during holidays. UniFi Access Requirement: 1.20.11 or later"""
    holiday_group_id: HolidayGroupId | None = None
    """Identity ID of the holiday group."""
    holiday_group: HolidayGroup | None = None


class FetchAllSchedulesResponse(_ScheduleBase):
    status: int
    """!!! note "not listed in API docs" """
    holiday_group_id: HolidayGroupId
    """!!! note "not listed in API docs" """
