import datetime
from typing import Annotated, Literal, NewType

from pydantic import (
    AwareDatetime,
    BaseModel,
    ConfigDict,
    PlainSerializer,
)


class ForbidExtraBaseModel(BaseModel):
    model_config = ConfigDict(extra="forbid")


# TODO: these are mostly UUIDs
AccessPolicyId = NewType("AccessPolicyId", str)
DoorId = NewType("DoorId", str)
FloorId = NewType("FloorId", str)
DoorGroupId = NewType("DoorGroupId", str)
ResourceId = DoorId | DoorGroupId
ScheduleId = NewType("ScheduleId", str)
UserId = NewType("UserId", str)
UserGroupId = NewType("UserGroupId", str)
VisitorId = NewType("VisitorId", str)
NfcCardId = NewType("NfcCardId", str)
HolidayId = NewType("HolidayId", str)
HolidayGroupId = NewType("HolidayGroupId", str)
DeviceId = NewType("DeviceId", str)
NfcCardEnrollmentSessionId = NewType("NfcCardEnrollmentSessionId", str)
NfcCardToken = NewType("NfcCardToken", str)
IdentityResourceId = NewType("IdentityResourceId", str)

ActorId = UserId | VisitorId | DeviceId

UnixTimestampDateTime = Annotated[
    AwareDatetime, PlainSerializer(lambda dt: int(dt.timestamp()), return_type=int)
]
RFC3339Datetime = Annotated[
    AwareDatetime,
    PlainSerializer(
        lambda dt: dt.astimezone(datetime.UTC).strftime("%Y-%m-%dT%H:%M:%SZ"),
        return_type=str,
    ),
]
FormattedTime = Annotated[
    datetime.time,
    PlainSerializer(lambda t: t.isoformat(timespec="seconds"), return_type=str),
]


class DoorResource(ForbidExtraBaseModel):
    type: Literal["door"] = "door"
    id: DoorId


class NamedDoorResource(DoorResource):
    name: str


class NamedDoorResourceWithIsBindHub(NamedDoorResource):
    is_bind_hub: bool


class DoorGroupResource(ForbidExtraBaseModel):
    type: Literal["door_group"] = "door_group"
    id: DoorGroupId


class NamedDoorGroupResource(DoorGroupResource):
    name: str


class NamedDoorGroupResourceWithIsBindHub(NamedDoorGroupResource):
    is_bind_hub: bool


Resource = DoorResource | DoorGroupResource
NamedResource = NamedDoorResource | NamedDoorGroupResource
NamedResourceWithIsBindHub = (
    NamedDoorResourceWithIsBindHub | NamedDoorGroupResourceWithIsBindHub
)


class PinCode(ForbidExtraBaseModel):
    token: str
    """The user's PIN hash code credential for unlocking a door."""
