from enum import StrEnum
from typing import Generic, Literal, TypeVar

from ._base import (
    DoorGroupId,
    DoorId,
    FloorId,
    ForbidExtraBaseModel,
    NamedResourceWithIsBindHub,
    Resource,
    UnixTimestampDateTime,
)


class DoorGroupType(StrEnum):
    ACCESS = "access"
    BUILDING = "building"


class ResourceTopology(ForbidExtraBaseModel):
    id: str
    """Identity ID of the floor."""
    type: str
    """Type of the floor."""
    name: str
    """Name of the floor."""
    resources: list[NamedResourceWithIsBindHub]
    """Contains all the doors on the floor."""
    is_bind_hub: bool | None = None
    """
    Indicate whether the door has bound to a hub device.
    It can only # be used for remote opening if it's bound.
    """


class DoorGroupTopology(ForbidExtraBaseModel):
    # XXX: is this really optional?
    id: str | None = None
    """Identity ID of the door group."""
    type: DoorGroupType
    """
    The building type contains all the doors; the access type represents all the customized door groups.
    """
    name: str
    """!!! note "not listed in API docs" """
    resource_topologies: list[ResourceTopology]
    """Contains information about the floor and all its associated doors."""


ResourceType = TypeVar("ResourceType", bound=Resource)


class DoorGroup(ForbidExtraBaseModel, Generic[ResourceType]):
    id: DoorGroupId
    """Identity ID of the door group."""
    name: str
    """Name of the door group."""
    type: DoorGroupType
    """!!! note "not listed in API docs" """
    resources: list[ResourceType]


class Door(ForbidExtraBaseModel):
    id: DoorId
    """Identity ID of the door."""
    name: str
    """Name of the door."""
    full_name: str
    """Full name of the door."""
    floor_id: FloorId
    """Identity ID of the floor."""
    # XXX: is this always 'door'?
    type: Literal["door"] = "door"
    """Type of the door."""
    is_bind_hub: bool
    """
    Indicate whether the door has bound to a hub device.
    It can only be used for remote opening if it's bound.
    !!! note "api doc describes this as a string"
    """
    door_lock_relay_status: Literal["lock", "unlock", ""]
    """Door lock status."""
    # TODO: convert empty string to None?
    door_position_status: Literal["open", "close", ""]
    """A null value means that no device is connected"""


class DoorLockingRuleType(StrEnum):
    KEEP_LOCK = "keep_lock"
    KEEP_UNLOCK = "keep_unlock"
    CUSTOM = "custom"
    RESET = "reset"
    LOCK_EARLY = "lock_early"


class DoorLockingRule(ForbidExtraBaseModel):
    type: DoorLockingRuleType
    ended_time: UnixTimestampDateTime


class DoorEmergencyStatus(ForbidExtraBaseModel):
    # NOTE: the api docs list these as not required, but always include them
    lockdown: bool = False
    evacuation: bool = False
