import datetime
from enum import StrEnum
from typing import Literal

from pydantic import Field

from ._base import ActorId, ForbidExtraBaseModel, UnixTimestampDateTime


class SystemLogTopic(StrEnum):
    ALL = "all"
    """All logs"""
    DOOR_OPENINGS = "door_openings"
    """Door opening logs"""
    CRITICAL = "critical"
    """Device restart, deletion, offline status, and detection"""
    UPDATES = "updates"
    """Device update logs"""
    DEVICE_EVENTS = "device_events"
    """
    Device online status, device updates, access policy synchronization,
    and active and inactive door unlock schedules
    """
    ADMIN_ACTIVITY = "admin_activity"
    """Admin activity, such as access policy updates, settings changes, and user management"""
    VISITOR = "visitor"
    """Visitor-related operations"""


class SystemLogEvent(ForbidExtraBaseModel):
    type: str
    display_message: str
    reason: str
    result: str
    published: UnixTimestampDateTime
    log_key: str


class SystemLogActor(ForbidExtraBaseModel):
    id: Literal[""] | ActorId  # XXX: check if this is complete
    type: str  # TODO: at least Literal["user", "service"], maybe more
    display_name: str
    alternate_id: str  # TODO: is this also an ActorId?
    alternate_name: str
    avatar: str | None = None
    sso_picture: str | None = None


class SystemLogAuthentication(ForbidExtraBaseModel):
    credential_provider: str
    issuer: str


# TODO: some of these could be narrowed
class SystemLogTarget(ForbidExtraBaseModel):
    type: str
    id: str
    alternate_id: str
    alternate_name: str
    display_name: str


class SystemLogSource(ForbidExtraBaseModel):
    actor: SystemLogActor
    event: SystemLogEvent
    authentication: SystemLogAuthentication | None
    target: list[SystemLogTarget]


class SystemLogEntry(ForbidExtraBaseModel):
    timestamp: datetime.datetime = Field(validation_alias="@timestamp")
    id: str = Field(alias="_id")
    source: SystemLogSource = Field(alias="_source")
    tag: str


class FetchSystemLogsResponse(ForbidExtraBaseModel):
    hits: list[SystemLogEntry]
