from enum import StrEnum
from typing import Annotated

from pydantic import BeforeValidator

from unifi_access.schemas import UserNfcCard
from unifi_access.schemas._access_policy import PartialSchedule

from ._base import (
    ForbidExtraBaseModel,
    NamedResource,
    PinCode,
    ScheduleId,
    UnixTimestampDateTime,
    UserId,
    VisitorId,
)


class VisitorStatus(StrEnum):
    UPCOMING = "UPCOMING"
    VISITED = "VISITED"
    VISITING = "VISITING"
    CANCELLED = "CANCELLED"
    NO_VISIT = "NO_VISIT"
    ACTIVE = "ACTIVE"


class VisitReason(StrEnum):
    INTERVIEW = "Interview"
    BUSINESS = "Business"
    COOPERATION = "Cooperation"
    OTHERS = "Others"


# TODO: some of these might not be nullable from some endpoints
# Generic to avoid duplication. `fetch_visitor` returns less info in schedule than `get_all_visitors`.
class Visitor(ForbidExtraBaseModel):
    id: VisitorId
    """Identity ID of the visitor."""
    first_name: str
    """First name of the visitor."""
    last_name: str
    """Last name of the visitor."""
    status: VisitorStatus
    """The visitor's status."""
    inviter_id: UserId | None = None
    """Identity ID of the inviter."""
    inviter_name: str | None = None
    """Name of the inviter."""
    mobile_phone: str | None = None
    """Contact phone number of the visitor."""
    remarks: str | None = None
    """Remarks of the visitor."""
    email: str | None = None
    """Email of the visitor."""
    visitor_company: str | None = None
    """Company of the visitor."""
    visit_reason: VisitReason | None = None
    """Visit reason"""
    start_time: UnixTimestampDateTime | None = None
    """Start time of the visit."""
    end_time: UnixTimestampDateTime | None = None
    """End time of the visit."""
    nfc_cards: list[UserNfcCard]

    # coerce empty dict to None, because for some reason that is what `fetch_visitor` returns
    pin_code: Annotated[PinCode | None, BeforeValidator(lambda v: v or None)] = None
    schedule_id: ScheduleId
    """Identity ID of the schedule."""
    schedule: PartialSchedule | None = None
    """
    The schedule assigned to the visitor.
    If the schedule information is present, it indicates that the visit schedule is recurring.
    If the schedule information is not included, it indicates a one-time visit schedule.
    """
    resources: list[NamedResource] | None = None
    """Specify the locations that the visitor can access."""
    avatar: str | None = None
    """!!! note "not listed in API docs" """
    location_id: str | None = None
    """!!! note "not listed in API docs" """


class FetchAllVisitorsExpansion(StrEnum):
    NONE = "none"
    ACCESS_POLICY = "access_policy"
    RESOURCE = "resource"
    SCHEDULE = "schedule"
    NFC_CARD = "nfc_card"
    PIN_CODE = "pin_code"
